/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-2001
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Chip Clark (chipc@netscape.com)
 *   Christopher Thomas (cst@yecc.com)
 *   Akihiro Misaki (spitfire.kuden@gmail.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ===== config.css ==============================================
   == Styles for about:config
   ======================================================================= */

@import url("chrome://global/skin/");
@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* ::::: warning screen ::::: */

#warningScreen
{
  background-color: #FFF;
}

#warningBox
{
  background-color: #C7D0D9;
  color: #22262F;
  border: 1px solid #494F5D;
  border-radius: 10px;
  padding: 3em;
  -moz-padding-start: 30px;
  margin: 0 1em;
}

#warningInnerBox
{
  max-width: 50em;
}

#warningTitle
{
  margin: 0 0 .6em;
  font-size: 160%;
  border-bottom: 1px solid #7A8490;
}

#warningText
{
  font-size: 110%;
  -moz-margin-start: 0;
}

#warningButton
{
  margin-top: 0.6em;
}

#showWarningNextTime
{
  margin-top: 0.6em;
}

#exclam
{
  width: 46px;
  height: 39px;
  list-style-image: url("chrome://global/skin/icons/alert-exclam.gif");
  -moz-margin-end: 3em;
}

/* ::::: tree rows ::::: */

treechildren::-moz-tree-cell-text(user)
{
  font-weight: bold;
}

treechildren::-moz-tree-cell-text(locked)
{
  font-style: italic;
}
