/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Joe Hewitt <hewitt@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ===== splitter.css ===================================================
  == Styles used by the XUL splitter element.
  ======================================================================= */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* ::::: splitter (vertical) ::::: */

splitter {
  -moz-box-align: center;
  -moz-box-pack: center;
  cursor: ew-resize;
  border-top: 1px solid;
  border-right: 2px solid;
  border-bottom: 1px solid;
  border-left: 2px solid;
  -moz-border-top-colors: #DFE5EF;
  -moz-border-right-colors: #3B414F #AAB4BF;
  -moz-border-bottom-colors: #AAB4BF;
  -moz-border-left-colors: #97A4B2 #DFE5EF;
  min-width: 5px;
  background-color: #C1CBD5;
}

splitter[state="collapsed"][collapse="before"],
splitter[state="collapsed"][substate="before"],
splitter[state="collapsed"][collapse="after"]:-moz-locale-dir(rtl),
splitter[state="collapsed"][substate="after"]:-moz-locale-dir(rtl) {
  cursor: e-resize;
}

splitter[state="collapsed"][collapse="after"],
splitter[state="collapsed"][substate="after"],
splitter[state="collapsed"][collapse="before"]:-moz-locale-dir(rtl),
splitter[state="collapsed"][substate="before"]:-moz-locale-dir(rtl) {
  cursor: w-resize;
}

/* ::::: splitter (horizontal) ::::: */

splitter[orient="vertical"] {
  cursor: ns-resize;
  border-top: 2px solid;
  border-right: 1px solid;
  border-bottom: 2px solid;
  border-left: 1px solid;
  -moz-border-top-colors: #97A4B2 #DFE5EF;
  -moz-border-right-colors: #AAB4BF;
  -moz-border-bottom-colors: #3B414F #AAB4BF;
  -moz-border-left-colors: #DFE5EF;
  min-width: 0;
  min-height: 5px;
}

splitter[orient="vertical"][state="collapsed"][collapse="before"],
splitter[orient="vertical"][state="collapsed"][substate="before"] {
  cursor: s-resize;
}

splitter[orient="vertical"][state="collapsed"][collapse="after"],
splitter[orient="vertical"][state="collapsed"][substate="after"] {
  cursor: n-resize;
}

splitter[disabled="true"] {
  cursor: default !important;
}

/* ::::: splitter grippy ::::: */

grippy {
  cursor: pointer;
  margin: 0px;
  min-width: 5px;
  min-height: 50px;
}

splitter[orient="vertical"] > grippy {
  margin: 0px;
  min-width: 50px;
  min-height: 5px;
}

/* ..... normal state ..... */

/* vertical grippies */
splitter[collapse="before"] > grippy {
  background-image: url("chrome://global/skin/splitter/grip-vrt-before.gif");
}

splitter[collapse="before"]:hover:active > grippy {
  background-image: url("chrome://global/skin/splitter/grip-vrt-before-act.gif");
}

splitter[collapse="before"] > grippy:hover:active {
  background-image: url("chrome://global/skin/splitter/grip-vrt-before-act.gif");
}

splitter[collapse="after"] > grippy {
  background-image: url("chrome://global/skin/splitter/grip-vrt-after.gif");
}

splitter[collapse="after"]:hover:active > grippy {
  background-image: url("chrome://global/skin/splitter/grip-vrt-after-act.gif");
}

splitter[collapse="after"] > grippy:hover:active {
  background-image: url("chrome://global/skin/splitter/grip-vrt-after-act.gif");
}

/* horizontal grippies */
splitter[collapse="before"][orient="vertical"] > grippy {
  background-image: url("chrome://global/skin/splitter/grip-hrz-before.gif");
}

splitter[collapse="before"][orient="vertical"]:hover:active > grippy {
  background-image: url("chrome://global/skin/splitter/grip-hrz-before-act.gif");
}

splitter[collapse="before"][orient="vertical"] > grippy:hover:active {
  background-image: url("chrome://global/skin/splitter/grip-hrz-before-act.gif");
}

splitter[collapse="after"][orient="vertical"] > grippy {
  background-image: url("chrome://global/skin/splitter/grip-hrz-after.gif");
}

splitter[collapse="after"][orient="vertical"]:hover:active > grippy {
  background-image: url("chrome://global/skin/splitter/grip-hrz-after-act.gif");
}

splitter[collapse="after"][orient="vertical"] > grippy:hover:active {
  background-image: url("chrome://global/skin/splitter/grip-hrz-after-act.gif");
}

/* ..... collapsed state ..... */

/* vertical grippies */
splitter[collapse="before"][state="collapsed"] > grippy {
  background-image: url("chrome://global/skin/splitter/grip-vrt-after.gif");
}

splitter[collapse="before"][state="collapsed"]:hover:active > grippy {
  background-image: url("chrome://global/skin/splitter/grip-vrt-after-act.gif");
}

splitter[collapse="before"][state="collapsed"] > grippy:hover:active {
  background-image: url("chrome://global/skin/splitter/grip-vrt-after-act.gif");
}

splitter[collapse="after"][state="collapsed"] > grippy {
  background-image: url("chrome://global/skin/splitter/grip-vrt-before.gif");
}

splitter[collapse="after"][state="collapsed"]:hover:active > grippy {
  background-image: url("chrome://global/skin/splitter/grip-vrt-before-act.gif");
}

splitter[collapse="after"][state="collapsed"] > grippy:hover:active {
  background-image: url("chrome://global/skin/splitter/grip-vrt-before-act.gif");
}

/* horizontal grippies */
splitter[collapse="before"][state="collapsed"][orient="vertical"] > grippy {
  background-image: url("chrome://global/skin/splitter/grip-hrz-after.gif");
}

splitter[collapse="before"][state="collapsed"][orient="vertical"]:hover:active > grippy {
  background-image: url("chrome://global/skin/splitter/grip-hrz-after-act.gif");
}

splitter[collapse="before"][state="collapsed"][orient="vertical"] > grippy:hover:active {
  background-image: url("chrome://global/skin/splitter/grip-hrz-after-act.gif");
}

splitter[collapse="after"][state="collapsed"][orient="vertical"] > grippy {
  background-image: url("chrome://global/skin/splitter/grip-hrz-before.gif");
}

splitter[collapse="after"][state="collapsed"][orient="vertical"]:hover:active > grippy {
  background-image: url("chrome://global/skin/splitter/grip-hrz-before-act.gif");
}

splitter[collapse="after"][state="collapsed"][orient="vertical"] > grippy:hover:active {
  background-image: url("chrome://global/skin/splitter/grip-hrz-before-act.gif");
}
