/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Oracle Corporation code.
 *
 * The Initial Developer of the Original Code is Oracle Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Vladimir Vukicevic <vladimir@pobox.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef GFX_GLITZSURFACE_H
#define GFX_GLITZSURFACE_H

#include "gfxASurface.h"

#include <cairo-glitz.h>

/**
 * A surface that wraps a glitz surface.
 */
class THEBES_API gfxGlitzSurface : public gfxASurface {
public:
    gfxGlitzSurface(glitz_drawable_t *drawable,
                    glitz_surface_t *glitzSurface,
                    bool takeOwnership = false);

    virtual ~gfxGlitzSurface();

    /**
     * When double-buffering is used, swaps the back and the front buffer.
     */
    void SwapBuffers();

    unsigned long Width();
    unsigned long Height();

    glitz_surface_t* GlitzSurface() { return mGlitzSurface; }
    glitz_drawable_t* GlitzDrawable() { return mGlitzDrawable; }

protected:
    glitz_drawable_t *mGlitzDrawable;
    glitz_surface_t *mGlitzSurface;
    bool mOwnsSurface;
};

#endif /* GFX_GLITZSURFACE_H */
