/*
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is Mozilla Communicator client code, released
# March 31, 1998.
#
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 1998-1999
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****
*/

.quick-search-button-image {
  padding: 0px;
}

/* it looks like appearance: searchfield gets us our search icon... */
#searchInput {
  margin: 3px 4px 4px !important;
  -moz-appearance: searchfield;
  font: icon;
  height: 22px !important;
}

#searchInput > .textbox-input-box {
  border: none;
  padding-top: 1px;
  height: 21px !important;
  margin: 0 !important;
}

#searchInput:-moz-lwtheme:not([focused="true"]) {
  opacity: .9;
}

/* things from xul.css that only exist if Thunderbird did not define
 * AUTOCOMPLETE_OLD_STYLE
 */

.autocomplete-history-dropmarker {
  display: none;
}

.autocomplete-history-dropmarker[enablehistory="true"] {
  display: -moz-box;
  -moz-binding: url("chrome://global/content/bindings/autocomplete.xml#history-dropmarker");
}
